<?php

namespace Drupal\domain_role_access\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\user\Entity\Role;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class DomainRolesForm.
 *
 * @package Drupal\domain_role_access\Form
 */
class DomainRolesForm extends FormBase {



  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */

  /**
   * The form constructor.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(MessengerInterface $messenger) {
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'domain_role_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $build_info = $form_state->getBuildInfo();

    /** @var \Drupal\domain\DomainInterface $domain */
    $domain = $build_info['args'][0];

    $options = [];

    $roles = Role::loadMultiple();

    foreach ($roles as $key => $role) {
      /** @var \Drupal\user\Entity\Role $role */
      $options[$key] = $role->label();
    }

    $config = $this->config('domain.roles.' . $domain->getOriginalId());

    $existing_roles = $config->get('roles') ?? [];

    $form = [
      'domain_id' => [
        '#type' => 'value',
        '#value' => $domain->getOriginalId(),
      ],
      'roles' => [
        '#type' => 'checkboxes',
        '#default_value' => array_keys($existing_roles),
        '#title' => $this->t('Roles'),
        '#options' => $options,
        '#description' => $this->t('These roles will have access to this domain.'),
      ],
      'actions' => [
        '#weight' => 20,
        '#type' => 'container',
        'submit' => [
          '#type' => 'submit',
          '#value' => $this->t('Submit'),
          '#button_type' => 'primary',
        ],
      ],
    ];

    $form['actions']['cancel'] = Link::createFromRoute($this->t('Cancel'), 'domain.admin')->toRenderable();
    $form['actions']['cancel']['#attributes']['class'] = ['button', 'button--danger'];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $domain_id = $form_state->getValue('domain_id');
    $roles = array_filter($form_state->getValue('roles'));
    $config = $this->configFactory->getEditable('domain.roles.' . $domain_id);
    if (empty($roles)) {
      $config->delete();
    }
    else {
      $config->set('roles', $roles);
      $config->save();
    }

    $this->messenger->addStatus($this->t('The configuration options have been saved.'));
  }
}
